/*
 * Copyright(C) Seiko Epson Corporation 2016. All rights reserved.
 *
 * Warranty Disclaimers.
 * You acknowledge and agree that the use of the software is at your own risk.
 * The software is provided "as is" and without any warranty of any kind.
 * Epson and its licensors do not and cannot warrant the performance or results
 * you may obtain by using the software.
 * Epson and its licensors make no warranties, express or implied, as to non-infringement,
 * merchantability or fitness for any particular purpose.
 */

package com.epson.emd.nmd.probt_2000sensors;

import android.app.Activity;
import android.btutil.KeyAssign;
import android.content.Intent;
import android.graphics.Matrix;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import org.w3c.dom.Text;

public class MainActivity extends Activity implements SensorEventListener {

    private SensorManager mSensorManager;
    private Sensor mAccelerometer = null;
    private ImageView mSchematicDiagram;
    private ImageButton mBtnRestart = null;
    private TextView mTxtrestart = null;
    private TextView mTxtMeasurements = null;
    private TextView mTxtSensorQuality = null;

    private RelativeLayout rlInstructions = null;
    private RelativeLayout rlSchemaDiagram = null;

    private static  final float MAXIMUM_X_VALUE = -400f;
    private static final float MAXIMUM_Y_VALUE = -650f;
    private static final float MOVE_RIGHT_LIMIT = -1.3f;
    private static final float MOVE_LEFT_LIMIT = 2.5f;
    private static final float MOVE_UP_LIMIT = 7.3f;
    private static final float MOVE_DOWN_LIMIT = 9.3f;
    private Matrix mMatrix;
    private boolean mPanActive = false;
    private boolean mIsFirstUsage = true;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_main);

        mSchematicDiagram = (ImageView) findViewById(R.id.imageView);

        rlInstructions = (RelativeLayout) findViewById(R.id.rlInstructions);

        rlSchemaDiagram = (RelativeLayout) findViewById(R.id.rlSchemaDiagram);

        mBtnRestart = (ImageButton)findViewById(R.id.btnRestart) ;

        mTxtrestart = (TextView)findViewById(R.id.txtRestart) ;

        mTxtMeasurements = (TextView)findViewById(R.id.txtMeasurements);

        mTxtSensorQuality = (TextView) findViewById(R.id.txtSensorQuality);

        mMatrix = new Matrix();

        mMatrix.setTranslate(MAXIMUM_X_VALUE / 2, MAXIMUM_Y_VALUE / 2);

        mSchematicDiagram.setImageMatrix(mMatrix);

        if (mAccelerometer == null){

            mSensorManager = (SensorManager)getSystemService(SENSOR_SERVICE);

            mAccelerometer = mSensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);

            mSensorManager.registerListener(this, mAccelerometer,  SensorManager.SENSOR_DELAY_FASTEST);
        }
    }

    /**
     * Button on click code to restart the demo and display the splash screen
     *
     * @param view
     */
    public void onClickStart(View view) {

        rlInstructions.setVisibility(View.GONE);

        mBtnRestart.setVisibility(View.GONE);

        mTxtrestart.setVisibility(View.GONE);

        rlSchemaDiagram.setVisibility(View.VISIBLE);

        KeyAssign.setKeyAssignMode (KeyAssign.KEYASSIGN_MODE_USER);
    }

    @Override
    public void onSensorChanged(SensorEvent event){

        String s = String.format("Axis Measurements : - X = %1$.3f, Y = %2$.3f, Z = %3$.3f Metres Per Second",
                event.values[0], event.values[1], event.values[2]);

        mTxtMeasurements.setText(s);



        if (event.accuracy > 2) {

            panSchematicImage(event.values);
        }

        switch (event.accuracy){
            case 1:
                mTxtSensorQuality.setText("Sensor Quality : - Poor");
            case 2:
                mTxtSensorQuality.setText("Sensor Quality : - Medium");
            case 3 :
                mTxtSensorQuality.setText("Sensor Quality : - High");
        }

    }

    private void panSchematicImage(float[] accelerometerValues) {

        if (mPanActive == true) {

            float[] matrix = new float[9];
            mMatrix.getValues(matrix);

            if (accelerometerValues[0] < MOVE_RIGHT_LIMIT) {

                if (matrix[2] < 100) {

                    mMatrix.setTranslate(matrix[2] + 50, matrix[5]);
                    mSchematicDiagram.setImageMatrix(mMatrix);
                }

                Log.d("EPSON_SENSOR", "Moving Right");

            } else if (accelerometerValues[0] > MOVE_LEFT_LIMIT) {

                if (matrix[2] > -800) {

                    mMatrix.setTranslate(matrix[2] - 50, matrix[5]);
                    mSchematicDiagram.setImageMatrix(mMatrix);
                }

                Log.d("EPSON_SENSOR", "Moving Left");
            }

            if (accelerometerValues[1] > MOVE_DOWN_LIMIT) {

                if (matrix[5] > -800) {

                    mMatrix.setTranslate(matrix[2], matrix[5] - 50);
                    mSchematicDiagram.setImageMatrix(mMatrix);
                }

                Log.d("EPSON_SENSOR", "Moving Down");

            } else if (accelerometerValues[1] < MOVE_UP_LIMIT) {

                if (matrix[5] != 125) {

                    mMatrix.setTranslate(matrix[2], matrix[5] + 50);
                    mSchematicDiagram.setImageMatrix(mMatrix);
                }

                Log.d("EPSON_SENSOR", "Moving Up");
            }
        }
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {

        if (mIsFirstUsage == true){

            super.onKeyDown(keyCode, event);

            mIsFirstUsage = false;

            return true;
        } else {

            if (keyCode == KeyEvent.KEYCODE_DPAD_CENTER){

                    mPanActive = true;

            } else if (keyCode == KeyEvent.KEYCODE_F1) {

                KeyAssign.setKeyAssignMode (KeyAssign.KEYASSIGN_MODE_DEFAULT);

                rlInstructions.setVisibility(View.VISIBLE);

                rlSchemaDiagram.setVisibility(View.GONE);

                mBtnRestart.setVisibility(View.VISIBLE);

                mTxtrestart.setVisibility(View.VISIBLE);

            }  else {

                return super.onKeyDown(keyCode, event);

            }
        }
        return true;
    }

    @Override
    public boolean onKeyUp(int keyCode, KeyEvent event) {

        switch (keyCode){
            case KeyEvent.KEYCODE_DPAD_CENTER:
                mPanActive = false;
                return true;
            default :
                return super.onKeyUp(keyCode, event);
        }
    }

    /**
     * Button on click code to restart the demo and display the splash screen
     *
     * @param view
     */
    public void onClickRestart(View view) {

        Intent i = getBaseContext().getPackageManager()
                .getLaunchIntentForPackage(getBaseContext().getPackageName());

        i.addFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);

        startActivity(i);
    }

    @Override
    public void onAccuracyChanged(Sensor sensor, int i) {
    }


    protected void onResume() {
        super.onResume();

        KeyAssign.setKeyAssignMode (KeyAssign.KEYASSIGN_MODE_DEFAULT);

        mSensorManager.registerListener(this, mAccelerometer,  SensorManager.SENSOR_DELAY_NORMAL);
    }

    protected void onPause() {
        super.onPause();

        mSensorManager.unregisterListener(this);
    }

}
